"""Type stubs for FILE_FILTER category."""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from hfortix_core.http.interface import IHTTPClient
    from .profile import Profile, ProfileDictMode, ProfileObjectMode

__all__ = [
    "Profile",
    "FilefilterDictMode",
    "FilefilterObjectMode",
]

class FilefilterDictMode:
    """FILE_FILTER API category for dict response mode.
    
    This class is returned when the client is instantiated with response_mode="dict" (default).
    All endpoints return dict/TypedDict responses by default.
    """
    
    profile: ProfileDictMode

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize file_filter category with HTTP client."""
        ...


class FilefilterObjectMode:
    """FILE_FILTER API category for object response mode.
    
    This class is returned when the client is instantiated with response_mode="object".
    All endpoints return FortiObject responses by default.
    """
    
    profile: ProfileObjectMode

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize file_filter category with HTTP client."""
        ...


# Base class for backwards compatibility
class Filefilter:
    """FILE_FILTER API category."""
    
    profile: Profile

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize file_filter category with HTTP client."""
        ...



"""FortiOS CMDB - Recommendation category"""

from .pse_config import PseConfig

__all__ = [
    "PseConfig",
    "Recommendation",
]


class Recommendation:
    """Recommendation endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """Recommendation endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.pse_config = PseConfig(client)



"""
Pydantic Models for CMDB - casb/saas_application

Runtime validation models for casb/saas_application configuration.
Generated from FortiOS schema version unknown.

Example Usage:
    >>> from hfortix_fortios.models.cmdb.casb.saas_application import 
    >>>
    >>> # Create with validation
    >>> obj = (
    ...     name=1,
    ...     name="example",
    ... )
    >>>
    >>> # Validation happens automatically
    >>> # ValidationError raised if constraints violated
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Literal, Optional

# ============================================================================
# Child Table Models
# ============================================================================

class SaasApplicationDomains(BaseModel):
    """
    Child table model for domains.
    
    SaaS application domain list.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
    
    domain: str = Field(max_length=127, default="", description="Domain list separated by space.")
class SaasApplicationOutputAttributes(BaseModel):
    """
    Child table model for output-attributes.
    
    SaaS application output attributes.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
    
    name: str | None = Field(max_length=79, default="", description="CASB attribute name.")    
    description: str | None = Field(max_length=63, default="", description="CASB attribute description.")    
    type: TypeEnum | None = Field(default="string", description="CASB attribute format type.")    
    optional: Literal["enable", "disable"] | None = Field(default="disable", description="CASB output attribute optional.")
class SaasApplicationInputAttributes(BaseModel):
    """
    Child table model for input-attributes.
    
    SaaS application input attributes.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
    
    name: str | None = Field(max_length=79, default="", description="CASB attribute name.")    
    description: str | None = Field(max_length=63, default="", description="CASB attribute description.")    
    type: Literal["string"] | None = Field(default="string", description="CASB attribute format type.")    
    required: Literal["enable", "disable"] | None = Field(default="enable", description="CASB input attribute required.")    
    default: Literal["string", "string-list"] | None = Field(default="string", description="CASB attribute default value.")    
    fallback_input: Literal["enable", "disable"] | None = Field(default="disable", description="CASB attribute legacy input.")
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================


# ============================================================================
# Main Model
# ============================================================================

class SaasApplicationModel(BaseModel):
    """
    Pydantic model for casb/saas_application configuration.
    
    Configure CASB SaaS application.
    
    Validation Rules:        - name: max_length=79 pattern=        - uuid: max_length=36 pattern=        - status: pattern=        - type: pattern=        - casb_name: max_length=79 pattern=        - description: max_length=63 pattern=        - domains: pattern=        - output_attributes: pattern=        - input_attributes: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    name: str | None = Field(max_length=79, default="", description="SaaS application name.")    
    uuid: str | None = Field(max_length=36, default="", description="Universally Unique Identifier (UUID; automatically assigned but can be manually reset).")    
    status: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable setting.")    
    type: Literal["built-in", "customized"] | None = Field(default="customized", description="SaaS application type.")    
    casb_name: str | None = Field(max_length=79, default="", description="SaaS application signature name.")    
    description: str | None = Field(max_length=63, default="", description="SaaS application description.")    
    domains: list[Domains] = Field(default=None, description="SaaS application domain list.")    
    output_attributes: list[OutputAttributes] = Field(default=None, description="SaaS application output attributes.")    
    input_attributes: list[InputAttributes] = Field(default=None, description="SaaS application input attributes.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "SaasApplicationModel",    "SaasApplicationDomains",    "SaasApplicationOutputAttributes",    "SaasApplicationInputAttributes",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-09T09:47:40.804171Z
# ============================================================================


"""Type stubs for SESSION category."""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from hfortix_core.http.interface import IHTTPClient
    from .cancel import Cancel, CancelDictMode, CancelObjectMode

__all__ = [
    "Cancel",
    "SessionDictMode",
    "SessionObjectMode",
]

class SessionDictMode:
    """SESSION API category for dict response mode.
    
    This class is returned when the client is instantiated with response_mode="dict" (default).
    All endpoints return dict/TypedDict responses by default.
    """
    
    cancel: CancelDictMode

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize session category with HTTP client."""
        ...


class SessionObjectMode:
    """SESSION API category for object response mode.
    
    This class is returned when the client is instantiated with response_mode="object".
    All endpoints return FortiObject responses by default.
    """
    
    cancel: CancelObjectMode

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize session category with HTTP client."""
        ...


# Base class for backwards compatibility
class Session:
    """SESSION API category."""
    
    cancel: Cancel

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize session category with HTTP client."""
        ...



"""Type stubs for CASB category."""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from hfortix_core.http.interface import IHTTPClient
    from .attribute_match import AttributeMatch, AttributeMatchDictMode, AttributeMatchObjectMode
    from .profile import Profile, ProfileDictMode, ProfileObjectMode
    from .saas_application import SaasApplication, SaasApplicationDictMode, SaasApplicationObjectMode
    from .user_activity import UserActivity, UserActivityDictMode, UserActivityObjectMode

__all__ = [
    "AttributeMatch",
    "Profile",
    "SaasApplication",
    "UserActivity",
    "CasbDictMode",
    "CasbObjectMode",
]

class CasbDictMode:
    """CASB API category for dict response mode.
    
    This class is returned when the client is instantiated with response_mode="dict" (default).
    All endpoints return dict/TypedDict responses by default.
    """
    
    attribute_match: AttributeMatchDictMode
    profile: ProfileDictMode
    saas_application: SaasApplicationDictMode
    user_activity: UserActivityDictMode

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize casb category with HTTP client."""
        ...


class CasbObjectMode:
    """CASB API category for object response mode.
    
    This class is returned when the client is instantiated with response_mode="object".
    All endpoints return FortiObject responses by default.
    """
    
    attribute_match: AttributeMatchObjectMode
    profile: ProfileObjectMode
    saas_application: SaasApplicationObjectMode
    user_activity: UserActivityObjectMode

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize casb category with HTTP client."""
        ...


# Base class for backwards compatibility
class Casb:
    """CASB API category."""
    
    attribute_match: AttributeMatch
    profile: Profile
    saas_application: SaasApplication
    user_activity: UserActivity

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize casb category with HTTP client."""
        ...



from typing import Any, Literal

# Enum type aliases for validation
VALID_BODY_LANGUAGE: Literal["english", "french", "spanish", "portuguese", "japanese", "trach", "simch", "korean"]
VALID_BODY_GUI_IPV6: Literal["enable", "disable"]
VALID_BODY_GUI_REPLACEMENT_MESSAGE_GROUPS: Literal["enable", "disable"]
VALID_BODY_GUI_LOCAL_OUT: Literal["enable", "disable"]
VALID_BODY_GUI_CERTIFICATES: Literal["enable", "disable"]
VALID_BODY_GUI_CUSTOM_LANGUAGE: Literal["enable", "disable"]
VALID_BODY_GUI_WIRELESS_OPENSECURITY: Literal["enable", "disable"]
VALID_BODY_GUI_APP_DETECTION_SDWAN: Literal["enable", "disable"]
VALID_BODY_GUI_DISPLAY_HOSTNAME: Literal["enable", "disable"]
VALID_BODY_GUI_FORTIGATE_CLOUD_SANDBOX: Literal["enable", "disable"]
VALID_BODY_GUI_FIRMWARE_UPGRADE_WARNING: Literal["enable", "disable"]
VALID_BODY_GUI_FORTICARE_REGISTRATION_SETUP_WARNING: Literal["enable", "disable"]
VALID_BODY_GUI_AUTO_UPGRADE_SETUP_WARNING: Literal["enable", "disable"]
VALID_BODY_GUI_WORKFLOW_MANAGEMENT: Literal["enable", "disable"]
VALID_BODY_GUI_CDN_USAGE: Literal["enable", "disable"]
VALID_BODY_ADMIN_HTTPS_SSL_VERSIONS: Literal["tlsv1-1", "tlsv1-2", "tlsv1-3"]
VALID_BODY_ADMIN_HTTPS_SSL_CIPHERSUITES: Literal["TLS-AES-128-GCM-SHA256", "TLS-AES-256-GCM-SHA384", "TLS-CHACHA20-POLY1305-SHA256", "TLS-AES-128-CCM-SHA256", "TLS-AES-128-CCM-8-SHA256"]
VALID_BODY_ADMIN_HTTPS_SSL_BANNED_CIPHERS: Literal["RSA", "DHE", "ECDHE", "DSS", "ECDSA", "AES", "AESGCM", "CAMELLIA", "3DES", "SHA1", "SHA256", "SHA384", "STATIC", "CHACHA20", "ARIA", "AESCCM"]
VALID_BODY_SSD_TRIM_FREQ: Literal["never", "hourly", "daily", "weekly", "monthly"]
VALID_BODY_SSD_TRIM_WEEKDAY: Literal["sunday", "monday", "tuesday", "wednesday", "thursday", "friday", "saturday"]
VALID_BODY_ADMIN_CONCURRENT: Literal["enable", "disable"]
VALID_BODY_PURDUE_LEVEL: Literal["1", "1.5", "2", "2.5", "3", "3.5", "4", "5", "5.5"]
VALID_BODY_DAILY_RESTART: Literal["enable", "disable"]
VALID_BODY_WAD_RESTART_MODE: Literal["none", "time", "memory"]
VALID_BODY_BATCH_CMDB: Literal["enable", "disable"]
VALID_BODY_MULTI_FACTOR_AUTHENTICATION: Literal["optional", "mandatory"]
VALID_BODY_SSL_MIN_PROTO_VERSION: Literal["SSLv3", "TLSv1", "TLSv1-1", "TLSv1-2", "TLSv1-3"]
VALID_BODY_AUTORUN_LOG_FSCK: Literal["enable", "disable"]
VALID_BODY_TRAFFIC_PRIORITY: Literal["tos", "dscp"]
VALID_BODY_TRAFFIC_PRIORITY_LEVEL: Literal["low", "medium", "high"]
VALID_BODY_QUIC_CONGESTION_CONTROL_ALGO: Literal["cubic", "bbr", "bbr2", "reno"]
VALID_BODY_QUIC_UDP_PAYLOAD_SIZE_SHAPING_PER_CID: Literal["enable", "disable"]
VALID_BODY_QUIC_PMTUD: Literal["enable", "disable"]
VALID_BODY_ANTI_REPLAY: Literal["disable", "loose", "strict"]
VALID_BODY_SEND_PMTU_ICMP: Literal["enable", "disable"]
VALID_BODY_HONOR_DF: Literal["enable", "disable"]
VALID_BODY_PMTU_DISCOVERY: Literal["enable", "disable"]
VALID_BODY_REVISION_IMAGE_AUTO_BACKUP: Literal["enable", "disable"]
VALID_BODY_REVISION_BACKUP_ON_LOGOUT: Literal["enable", "disable"]
VALID_BODY_STRONG_CRYPTO: Literal["enable", "disable"]
VALID_BODY_SSL_STATIC_KEY_CIPHERS: Literal["enable", "disable"]
VALID_BODY_SNAT_ROUTE_CHANGE: Literal["enable", "disable"]
VALID_BODY_IPV6_SNAT_ROUTE_CHANGE: Literal["enable", "disable"]
VALID_BODY_SPEEDTEST_SERVER: Literal["enable", "disable"]
VALID_BODY_CLI_AUDIT_LOG: Literal["enable", "disable"]
VALID_BODY_DH_PARAMS: Literal["1024", "1536", "2048", "3072", "4096", "6144", "8192"]
VALID_BODY_FDS_STATISTICS: Literal["enable", "disable"]
VALID_BODY_TCP_OPTION: Literal["enable", "disable"]
VALID_BODY_LLDP_TRANSMISSION: Literal["enable", "disable"]
VALID_BODY_LLDP_RECEPTION: Literal["enable", "disable"]
VALID_BODY_PROXY_KEEP_ALIVE_MODE: Literal["session", "traffic", "re-authentication"]
VALID_BODY_PROXY_AUTH_LIFETIME: Literal["enable", "disable"]
VALID_BODY_PROXY_RESOURCE_MODE: Literal["enable", "disable"]
VALID_BODY_PROXY_CERT_USE_MGMT_VDOM: Literal["enable", "disable"]
VALID_BODY_CHECK_PROTOCOL_HEADER: Literal["loose", "strict"]
VALID_BODY_VIP_ARP_RANGE: Literal["unlimited", "restricted"]
VALID_BODY_RESET_SESSIONLESS_TCP: Literal["enable", "disable"]
VALID_BODY_ALLOW_TRAFFIC_REDIRECT: Literal["enable", "disable"]
VALID_BODY_IPV6_ALLOW_TRAFFIC_REDIRECT: Literal["enable", "disable"]
VALID_BODY_STRICT_DIRTY_SESSION_CHECK: Literal["enable", "disable"]
VALID_BODY_PRE_LOGIN_BANNER: Literal["enable", "disable"]
VALID_BODY_POST_LOGIN_BANNER: Literal["disable", "enable"]
VALID_BODY_TFTP: Literal["enable", "disable"]
VALID_BODY_AV_FAILOPEN: Literal["pass", "off", "one-shot"]
VALID_BODY_AV_FAILOPEN_SESSION: Literal["enable", "disable"]
VALID_BODY_LOG_SINGLE_CPU_HIGH: Literal["enable", "disable"]
VALID_BODY_CHECK_RESET_RANGE: Literal["strict", "disable"]
VALID_BODY_UPGRADE_REPORT: Literal["enable", "disable"]
VALID_BODY_ADMIN_HTTPS_REDIRECT: Literal["enable", "disable"]
VALID_BODY_ADMIN_SSH_PASSWORD: Literal["enable", "disable"]
VALID_BODY_ADMIN_RESTRICT_LOCAL: Literal["all", "non-console-only", "disable"]
VALID_BODY_ADMIN_SSH_V1: Literal["enable", "disable"]
VALID_BODY_ADMIN_TELNET: Literal["enable", "disable"]
VALID_BODY_ADMIN_FORTICLOUD_SSO_LOGIN: Literal["enable", "disable"]
VALID_BODY_ADMIN_HTTPS_PKI_REQUIRED: Literal["enable", "disable"]
VALID_BODY_AUTH_KEEPALIVE: Literal["enable", "disable"]
VALID_BODY_AUTH_SESSION_LIMIT: Literal["block-new", "logout-inactive"]
VALID_BODY_CLT_CERT_REQ: Literal["enable", "disable"]
VALID_BODY_CFG_SAVE: Literal["automatic", "manual", "revert"]
VALID_BODY_REBOOT_UPON_CONFIG_RESTORE: Literal["enable", "disable"]
VALID_BODY_ADMIN_SCP: Literal["enable", "disable"]
VALID_BODY_WIRELESS_CONTROLLER: Literal["enable", "disable"]
VALID_BODY_FORTIEXTENDER: Literal["disable", "enable"]
VALID_BODY_FORTIEXTENDER_DISCOVERY_LOCKDOWN: Literal["disable", "enable"]
VALID_BODY_FORTIEXTENDER_VLAN_MODE: Literal["enable", "disable"]
VALID_BODY_FORTIEXTENDER_PROVISION_ON_AUTHORIZATION: Literal["enable", "disable"]
VALID_BODY_SWITCH_CONTROLLER: Literal["disable", "enable"]
VALID_BODY_FGD_ALERT_SUBSCRIPTION: Literal["advisory", "latest-threat", "latest-virus", "latest-attack", "new-antivirus-db", "new-attack-db"]
VALID_BODY_IPV6_ALLOW_ANYCAST_PROBE: Literal["enable", "disable"]
VALID_BODY_IPV6_ALLOW_MULTICAST_PROBE: Literal["enable", "disable"]
VALID_BODY_IPV6_ALLOW_LOCAL_IN_SILENT_DROP: Literal["enable", "disable"]
VALID_BODY_CSR_CA_ATTRIBUTE: Literal["enable", "disable"]
VALID_BODY_WIMAX_4G_USB: Literal["enable", "disable"]
VALID_BODY_SSLVPN_WEB_MODE: Literal["enable", "disable"]
VALID_BODY_PER_USER_BAL: Literal["enable", "disable"]
VALID_BODY_WAD_SOURCE_AFFINITY: Literal["disable", "enable"]
VALID_BODY_LOGIN_TIMESTAMP: Literal["enable", "disable"]
VALID_BODY_IP_CONFLICT_DETECTION: Literal["enable", "disable"]
VALID_BODY_SPECIAL_FILE_23_SUPPORT: Literal["disable", "enable"]
VALID_BODY_LOG_UUID_ADDRESS: Literal["enable", "disable"]
VALID_BODY_LOG_SSL_CONNECTION: Literal["enable", "disable"]
VALID_BODY_GUI_REST_API_CACHE: Literal["enable", "disable"]
VALID_BODY_REST_API_KEY_URL_QUERY: Literal["enable", "disable"]
VALID_BODY_IPSEC_QAT_OFFLOAD: Literal["enable", "disable"]
VALID_BODY_PRIVATE_DATA_ENCRYPTION: Literal["disable", "enable"]
VALID_BODY_AUTO_AUTH_EXTENSION_DEVICE: Literal["enable", "disable"]
VALID_BODY_GUI_THEME: Literal["jade", "neutrino", "mariner", "graphite", "melongene", "jet-stream", "security-fabric", "retro", "dark-matter", "onyx", "eclipse"]
VALID_BODY_GUI_DATE_FORMAT: Literal["yyyy/MM/dd", "dd/MM/yyyy", "MM/dd/yyyy", "yyyy-MM-dd", "dd-MM-yyyy", "MM-dd-yyyy"]
VALID_BODY_GUI_DATE_TIME_SOURCE: Literal["system", "browser"]
VALID_BODY_CLOUD_COMMUNICATION: Literal["enable", "disable"]
VALID_BODY_FORTITOKEN_CLOUD: Literal["enable", "disable"]
VALID_BODY_FORTITOKEN_CLOUD_PUSH_STATUS: Literal["enable", "disable"]
VALID_BODY_IRQ_TIME_ACCOUNTING: Literal["auto", "force"]
VALID_BODY_MANAGEMENT_PORT_USE_ADMIN_SPORT: Literal["enable", "disable"]
VALID_BODY_FORTICONVERTER_INTEGRATION: Literal["enable", "disable"]
VALID_BODY_FORTICONVERTER_CONFIG_UPLOAD: Literal["once", "disable"]
VALID_BODY_INTERNET_SERVICE_DATABASE: Literal["mini", "standard", "full", "on-demand"]
VALID_BODY_GEOIP_FULL_DB: Literal["enable", "disable"]
VALID_BODY_EARLY_TCP_NPU_SESSION: Literal["enable", "disable"]
VALID_BODY_NPU_NEIGHBOR_UPDATE: Literal["enable", "disable"]
VALID_BODY_DELAY_TCP_NPU_SESSION: Literal["enable", "disable"]
VALID_BODY_INTERFACE_SUBNET_USAGE: Literal["disable", "enable"]
VALID_BODY_FORTIGSLB_INTEGRATION: Literal["disable", "enable"]
VALID_BODY_AUTH_SESSION_AUTO_BACKUP: Literal["enable", "disable"]
VALID_BODY_AUTH_SESSION_AUTO_BACKUP_INTERVAL: Literal["1min", "5min", "15min", "30min", "1hr"]
VALID_BODY_APPLICATION_BANDWIDTH_TRACKING: Literal["disable", "enable"]
VALID_BODY_TLS_SESSION_CACHE: Literal["enable", "disable"]

# Metadata dictionaries
FIELD_TYPES: dict[str, str]
FIELD_DESCRIPTIONS: dict[str, str]
FIELD_CONSTRAINTS: dict[str, dict[str, Any]]
NESTED_SCHEMAS: dict[str, dict[str, Any]]
FIELDS_WITH_DEFAULTS: dict[str, Any]

# Helper functions
def get_field_type(field_name: str) -> str | None: ...
def get_field_description(field_name: str) -> str | None: ...
def get_field_default(field_name: str) -> Any: ...
def get_field_constraints(field_name: str) -> dict[str, Any]: ...
def get_nested_schema(field_name: str) -> dict[str, Any] | None: ...
def get_field_metadata(field_name: str) -> dict[str, Any]: ...
def validate_field_value(field_name: str, value: Any) -> bool: ...
def get_all_fields() -> list[str]: ...
def get_required_fields() -> list[str]: ...
def get_schema_info() -> dict[str, Any]: ...


__all__ = [
    "VALID_BODY_LANGUAGE",
    "VALID_BODY_GUI_IPV6",
    "VALID_BODY_GUI_REPLACEMENT_MESSAGE_GROUPS",
    "VALID_BODY_GUI_LOCAL_OUT",
    "VALID_BODY_GUI_CERTIFICATES",
    "VALID_BODY_GUI_CUSTOM_LANGUAGE",
    "VALID_BODY_GUI_WIRELESS_OPENSECURITY",
    "VALID_BODY_GUI_APP_DETECTION_SDWAN",
    "VALID_BODY_GUI_DISPLAY_HOSTNAME",
    "VALID_BODY_GUI_FORTIGATE_CLOUD_SANDBOX",
    "VALID_BODY_GUI_FIRMWARE_UPGRADE_WARNING",
    "VALID_BODY_GUI_FORTICARE_REGISTRATION_SETUP_WARNING",
    "VALID_BODY_GUI_AUTO_UPGRADE_SETUP_WARNING",
    "VALID_BODY_GUI_WORKFLOW_MANAGEMENT",
    "VALID_BODY_GUI_CDN_USAGE",
    "VALID_BODY_ADMIN_HTTPS_SSL_VERSIONS",
    "VALID_BODY_ADMIN_HTTPS_SSL_CIPHERSUITES",
    "VALID_BODY_ADMIN_HTTPS_SSL_BANNED_CIPHERS",
    "VALID_BODY_SSD_TRIM_FREQ",
    "VALID_BODY_SSD_TRIM_WEEKDAY",
    "VALID_BODY_ADMIN_CONCURRENT",
    "VALID_BODY_PURDUE_LEVEL",
    "VALID_BODY_DAILY_RESTART",
    "VALID_BODY_WAD_RESTART_MODE",
    "VALID_BODY_BATCH_CMDB",
    "VALID_BODY_MULTI_FACTOR_AUTHENTICATION",
    "VALID_BODY_SSL_MIN_PROTO_VERSION",
    "VALID_BODY_AUTORUN_LOG_FSCK",
    "VALID_BODY_TRAFFIC_PRIORITY",
    "VALID_BODY_TRAFFIC_PRIORITY_LEVEL",
    "VALID_BODY_QUIC_CONGESTION_CONTROL_ALGO",
    "VALID_BODY_QUIC_UDP_PAYLOAD_SIZE_SHAPING_PER_CID",
    "VALID_BODY_QUIC_PMTUD",
    "VALID_BODY_ANTI_REPLAY",
    "VALID_BODY_SEND_PMTU_ICMP",
    "VALID_BODY_HONOR_DF",
    "VALID_BODY_PMTU_DISCOVERY",
    "VALID_BODY_REVISION_IMAGE_AUTO_BACKUP",
    "VALID_BODY_REVISION_BACKUP_ON_LOGOUT",
    "VALID_BODY_STRONG_CRYPTO",
    "VALID_BODY_SSL_STATIC_KEY_CIPHERS",
    "VALID_BODY_SNAT_ROUTE_CHANGE",
    "VALID_BODY_IPV6_SNAT_ROUTE_CHANGE",
    "VALID_BODY_SPEEDTEST_SERVER",
    "VALID_BODY_CLI_AUDIT_LOG",
    "VALID_BODY_DH_PARAMS",
    "VALID_BODY_FDS_STATISTICS",
    "VALID_BODY_TCP_OPTION",
    "VALID_BODY_LLDP_TRANSMISSION",
    "VALID_BODY_LLDP_RECEPTION",
    "VALID_BODY_PROXY_KEEP_ALIVE_MODE",
    "VALID_BODY_PROXY_AUTH_LIFETIME",
    "VALID_BODY_PROXY_RESOURCE_MODE",
    "VALID_BODY_PROXY_CERT_USE_MGMT_VDOM",
    "VALID_BODY_CHECK_PROTOCOL_HEADER",
    "VALID_BODY_VIP_ARP_RANGE",
    "VALID_BODY_RESET_SESSIONLESS_TCP",
    "VALID_BODY_ALLOW_TRAFFIC_REDIRECT",
    "VALID_BODY_IPV6_ALLOW_TRAFFIC_REDIRECT",
    "VALID_BODY_STRICT_DIRTY_SESSION_CHECK",
    "VALID_BODY_PRE_LOGIN_BANNER",
    "VALID_BODY_POST_LOGIN_BANNER",
    "VALID_BODY_TFTP",
    "VALID_BODY_AV_FAILOPEN",
    "VALID_BODY_AV_FAILOPEN_SESSION",
    "VALID_BODY_LOG_SINGLE_CPU_HIGH",
    "VALID_BODY_CHECK_RESET_RANGE",
    "VALID_BODY_UPGRADE_REPORT",
    "VALID_BODY_ADMIN_HTTPS_REDIRECT",
    "VALID_BODY_ADMIN_SSH_PASSWORD",
    "VALID_BODY_ADMIN_RESTRICT_LOCAL",
    "VALID_BODY_ADMIN_SSH_V1",
    "VALID_BODY_ADMIN_TELNET",
    "VALID_BODY_ADMIN_FORTICLOUD_SSO_LOGIN",
    "VALID_BODY_ADMIN_HTTPS_PKI_REQUIRED",
    "VALID_BODY_AUTH_KEEPALIVE",
    "VALID_BODY_AUTH_SESSION_LIMIT",
    "VALID_BODY_CLT_CERT_REQ",
    "VALID_BODY_CFG_SAVE",
    "VALID_BODY_REBOOT_UPON_CONFIG_RESTORE",
    "VALID_BODY_ADMIN_SCP",
    "VALID_BODY_WIRELESS_CONTROLLER",
    "VALID_BODY_FORTIEXTENDER",
    "VALID_BODY_FORTIEXTENDER_DISCOVERY_LOCKDOWN",
    "VALID_BODY_FORTIEXTENDER_VLAN_MODE",
    "VALID_BODY_FORTIEXTENDER_PROVISION_ON_AUTHORIZATION",
    "VALID_BODY_SWITCH_CONTROLLER",
    "VALID_BODY_FGD_ALERT_SUBSCRIPTION",
    "VALID_BODY_IPV6_ALLOW_ANYCAST_PROBE",
    "VALID_BODY_IPV6_ALLOW_MULTICAST_PROBE",
    "VALID_BODY_IPV6_ALLOW_LOCAL_IN_SILENT_DROP",
    "VALID_BODY_CSR_CA_ATTRIBUTE",
    "VALID_BODY_WIMAX_4G_USB",
    "VALID_BODY_SSLVPN_WEB_MODE",
    "VALID_BODY_PER_USER_BAL",
    "VALID_BODY_WAD_SOURCE_AFFINITY",
    "VALID_BODY_LOGIN_TIMESTAMP",
    "VALID_BODY_IP_CONFLICT_DETECTION",
    "VALID_BODY_SPECIAL_FILE_23_SUPPORT",
    "VALID_BODY_LOG_UUID_ADDRESS",
    "VALID_BODY_LOG_SSL_CONNECTION",
    "VALID_BODY_GUI_REST_API_CACHE",
    "VALID_BODY_REST_API_KEY_URL_QUERY",
    "VALID_BODY_IPSEC_QAT_OFFLOAD",
    "VALID_BODY_PRIVATE_DATA_ENCRYPTION",
    "VALID_BODY_AUTO_AUTH_EXTENSION_DEVICE",
    "VALID_BODY_GUI_THEME",
    "VALID_BODY_GUI_DATE_FORMAT",
    "VALID_BODY_GUI_DATE_TIME_SOURCE",
    "VALID_BODY_CLOUD_COMMUNICATION",
    "VALID_BODY_FORTITOKEN_CLOUD",
    "VALID_BODY_FORTITOKEN_CLOUD_PUSH_STATUS",
    "VALID_BODY_IRQ_TIME_ACCOUNTING",
    "VALID_BODY_MANAGEMENT_PORT_USE_ADMIN_SPORT",
    "VALID_BODY_FORTICONVERTER_INTEGRATION",
    "VALID_BODY_FORTICONVERTER_CONFIG_UPLOAD",
    "VALID_BODY_INTERNET_SERVICE_DATABASE",
    "VALID_BODY_GEOIP_FULL_DB",
    "VALID_BODY_EARLY_TCP_NPU_SESSION",
    "VALID_BODY_NPU_NEIGHBOR_UPDATE",
    "VALID_BODY_DELAY_TCP_NPU_SESSION",
    "VALID_BODY_INTERFACE_SUBNET_USAGE",
    "VALID_BODY_FORTIGSLB_INTEGRATION",
    "VALID_BODY_AUTH_SESSION_AUTO_BACKUP",
    "VALID_BODY_AUTH_SESSION_AUTO_BACKUP_INTERVAL",
    "VALID_BODY_APPLICATION_BANDWIDTH_TRACKING",
    "VALID_BODY_TLS_SESSION_CACHE",
    "FIELD_TYPES",
    "FIELD_DESCRIPTIONS",
    "FIELD_CONSTRAINTS",
    "NESTED_SCHEMAS",
    "FIELDS_WITH_DEFAULTS",
    "get_field_type",
    "get_field_description",
    "get_field_default",
    "get_field_constraints",
    "get_nested_schema",
    "get_field_metadata",
    "validate_field_value",
    "get_all_fields",
    "get_required_fields",
    "get_schema_info",
]


"""
Pydantic Models for CMDB - system/dhcp/server

Runtime validation models for system/dhcp/server configuration.
Generated from FortiOS schema version unknown.

Example Usage:
    >>> from hfortix_fortios.models.cmdb.system.dhcp.server import 
    >>>
    >>> # Create with validation
    >>> obj = (
    ...     id=1,
    ...     name="example",
    ... )
    >>>
    >>> # Validation happens automatically
    >>> # ValidationError raised if constraints violated
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Literal, Optional

# ============================================================================
# Child Table Models
# ============================================================================

class ServerIpRange(BaseModel):
    """
    Child table model for ip-range.
    
    DHCP IP range configuration.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
    
    id: int = Field(ge=0, le=4294967295, default=0, description="ID.")    
    start_ip: str = Field(default="0.0.0.0", description="Start of IP range.")    
    end_ip: str = Field(default="0.0.0.0", description="End of IP range.")    
    vci_match: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable vendor class identifier (VCI) matching. When enabled only DHCP requests with a matching VCI are served with this range.")    
    vci_string: list[VciString] = Field(default=None, description="One or more VCI strings in quotes separated by spaces.")    
    uci_match: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable user class identifier (UCI) matching. When enabled only DHCP requests with a matching UCI are served with this range.")    
    uci_string: list[UciString] = Field(default=None, description="One or more UCI strings in quotes separated by spaces.")    
    lease_time: int | None = Field(ge=300, le=8640000, default=0, description="Lease time in seconds, 0 means default lease time.")
class ServerTftpServer(BaseModel):
    """
    Child table model for tftp-server.
    
    One or more hostnames or IP addresses of the TFTP servers in quotes separated by spaces.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
    
    tftp_server: str = Field(max_length=63, default="", description="TFTP server.")
class ServerOptions(BaseModel):
    """
    Child table model for options.
    
    DHCP options.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
    
    id: int = Field(ge=0, le=4294967295, default=0, description="ID.")    
    code: int = Field(ge=0, le=255, default=0, description="DHCP option code.")    
    type: TypeEnum | None = Field(default="hex", description="DHCP option type.")    
    value: str | None = Field(max_length=312, default="", description="DHCP option value.")    
    ip: list[Ip] = Field(default="", description="DHCP option IPs.")    
    vci_match: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable vendor class identifier (VCI) matching. When enabled only DHCP requests with a matching VCI are served with this option.")    
    vci_string: list[VciString] = Field(default=None, description="One or more VCI strings in quotes separated by spaces.")    
    uci_match: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable user class identifier (UCI) matching. When enabled only DHCP requests with a matching UCI are served with this option.")    
    uci_string: list[UciString] = Field(default=None, description="One or more UCI strings in quotes separated by spaces.")
class ServerVciString(BaseModel):
    """
    Child table model for vci-string.
    
    One or more VCI strings in quotes separated by spaces.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
    
    vci_string: str = Field(max_length=255, default="", description="VCI strings.")
class ServerExcludeRange(BaseModel):
    """
    Child table model for exclude-range.
    
    Exclude one or more ranges of IP addresses from being assigned to clients.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
    
    id: int = Field(ge=0, le=4294967295, default=0, description="ID.")    
    start_ip: str = Field(default="0.0.0.0", description="Start of IP range.")    
    end_ip: str = Field(default="0.0.0.0", description="End of IP range.")    
    vci_match: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable vendor class identifier (VCI) matching. When enabled only DHCP requests with a matching VCI are served with this range.")    
    vci_string: list[VciString] = Field(default=None, description="One or more VCI strings in quotes separated by spaces.")    
    uci_match: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable user class identifier (UCI) matching. When enabled only DHCP requests with a matching UCI are served with this range.")    
    uci_string: list[UciString] = Field(default=None, description="One or more UCI strings in quotes separated by spaces.")    
    lease_time: int | None = Field(ge=300, le=8640000, default=0, description="Lease time in seconds, 0 means default lease time.")
class ServerReservedAddress(BaseModel):
    """
    Child table model for reserved-address.
    
    Options for the DHCP server to assign IP settings to specific MAC addresses.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
    
    id: int = Field(ge=0, le=4294967295, default=0, description="ID.")    
    type: Literal["mac", "option82"] | None = Field(default="mac", description="DHCP reserved-address type.")    
    ip: str = Field(default="0.0.0.0", description="IP address to be reserved for the MAC address.")    
    mac: str = Field(default="00:00:00:00:00:00", description="MAC address of the client that will get the reserved IP address.")    
    action: Literal["assign", "block", "reserved"] | None = Field(default="reserved", description="Options for the DHCP server to configure the client with the reserved MAC address.")    
    circuit_id_type: Literal["hex", "string"] | None = Field(default="string", description="DHCP option type.")    
    circuit_id: str | None = Field(max_length=312, default="", description="Option 82 circuit-ID of the client that will get the reserved IP address.")    
    remote_id_type: Literal["hex", "string"] | None = Field(default="string", description="DHCP option type.")    
    remote_id: str | None = Field(max_length=312, default="", description="Option 82 remote-ID of the client that will get the reserved IP address.")    
    description: str | None = Field(max_length=255, default=None, description="Description.")
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================


# ============================================================================
# Main Model
# ============================================================================

class ServerModel(BaseModel):
    """
    Pydantic model for system/dhcp/server configuration.
    
    Configure DHCP servers.
    
    Validation Rules:        - id: min=0 max=4294967295 pattern=        - status: pattern=        - lease_time: min=300 max=8640000 pattern=        - mac_acl_default_action: pattern=        - forticlient_on_net_status: pattern=        - dns_service: pattern=        - dns_server1: pattern=        - dns_server2: pattern=        - dns_server3: pattern=        - dns_server4: pattern=        - wifi_ac_service: pattern=        - wifi_ac1: pattern=        - wifi_ac2: pattern=        - wifi_ac3: pattern=        - ntp_service: pattern=        - ntp_server1: pattern=        - ntp_server2: pattern=        - ntp_server3: pattern=        - domain: max_length=35 pattern=        - wins_server1: pattern=        - wins_server2: pattern=        - default_gateway: pattern=        - next_server: pattern=        - netmask: pattern=        - interface: max_length=15 pattern=        - ip_range: pattern=        - timezone_option: pattern=        - timezone: max_length=63 pattern=        - tftp_server: pattern=        - filename: max_length=127 pattern=        - options: pattern=        - server_type: pattern=        - ip_mode: pattern=        - conflicted_ip_timeout: min=60 max=8640000 pattern=        - ipsec_lease_hold: min=0 max=8640000 pattern=        - auto_configuration: pattern=        - dhcp_settings_from_fortiipam: pattern=        - auto_managed_status: pattern=        - ddns_update: pattern=        - ddns_update_override: pattern=        - ddns_server_ip: pattern=        - ddns_zone: max_length=64 pattern=        - ddns_auth: pattern=        - ddns_keyname: max_length=64 pattern=        - ddns_key: pattern=        - ddns_ttl: min=60 max=86400 pattern=        - vci_match: pattern=        - vci_string: pattern=        - exclude_range: pattern=        - shared_subnet: pattern=        - relay_agent: pattern=        - reserved_address: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    id: int = Field(ge=0, le=4294967295, default=0, description="ID.")    
    status: Literal["disable", "enable"] | None = Field(default="enable", description="Enable/disable this DHCP configuration.")    
    lease_time: int | None = Field(ge=300, le=8640000, default=604800, description="Lease time in seconds, 0 means unlimited.")    
    mac_acl_default_action: Literal["assign", "block"] | None = Field(default="assign", description="MAC access control default action (allow or block assigning IP settings).")    
    forticlient_on_net_status: Literal["disable", "enable"] | None = Field(default="enable", description="Enable/disable FortiClient-On-Net service for this DHCP server.")    
    dns_service: Literal["local", "default", "specify"] | None = Field(default="specify", description="Options for assigning DNS servers to DHCP clients.")    
    dns_server1: str | None = Field(default="0.0.0.0", description="DNS server 1.")    
    dns_server2: str | None = Field(default="0.0.0.0", description="DNS server 2.")    
    dns_server3: str | None = Field(default="0.0.0.0", description="DNS server 3.")    
    dns_server4: str | None = Field(default="0.0.0.0", description="DNS server 4.")    
    wifi_ac_service: Literal["specify", "local"] | None = Field(default="specify", description="Options for assigning WiFi access controllers to DHCP clients.")    
    wifi_ac1: str | None = Field(default="0.0.0.0", description="WiFi Access Controller 1 IP address (DHCP option 138, RFC 5417).")    
    wifi_ac2: str | None = Field(default="0.0.0.0", description="WiFi Access Controller 2 IP address (DHCP option 138, RFC 5417).")    
    wifi_ac3: str | None = Field(default="0.0.0.0", description="WiFi Access Controller 3 IP address (DHCP option 138, RFC 5417).")    
    ntp_service: Literal["local", "default", "specify"] | None = Field(default="specify", description="Options for assigning Network Time Protocol (NTP) servers to DHCP clients.")    
    ntp_server1: str | None = Field(default="0.0.0.0", description="NTP server 1.")    
    ntp_server2: str | None = Field(default="0.0.0.0", description="NTP server 2.")    
    ntp_server3: str | None = Field(default="0.0.0.0", description="NTP server 3.")    
    domain: str | None = Field(max_length=35, default="", description="Domain name suffix for the IP addresses that the DHCP server assigns to clients.")    
    wins_server1: str | None = Field(default="0.0.0.0", description="WINS server 1.")    
    wins_server2: str | None = Field(default="0.0.0.0", description="WINS server 2.")    
    default_gateway: str | None = Field(default="0.0.0.0", description="Default gateway IP address assigned by the DHCP server.")    
    next_server: str | None = Field(default="0.0.0.0", description="IP address of a server (for example, a TFTP sever) that DHCP clients can download a boot file from.")    
    netmask: str = Field(default="0.0.0.0", description="Netmask assigned by the DHCP server.")    
    interface: str = Field(max_length=15, default="", description="DHCP server can assign IP configurations to clients connected to this interface.")  # datasource: ['system.interface.name']    
    ip_range: list[IpRange] = Field(default=None, description="DHCP IP range configuration.")    
    timezone_option: Literal["disable", "default", "specify"] | None = Field(default="disable", description="Options for the DHCP server to set the client's time zone.")    
    timezone: str = Field(max_length=63, default="", description="Select the time zone to be assigned to DHCP clients.")  # datasource: ['system.timezone.name']    
    tftp_server: list[TftpServer] = Field(default=None, description="One or more hostnames or IP addresses of the TFTP servers in quotes separated by spaces.")    
    filename: str | None = Field(max_length=127, default="", description="Name of the boot file on the TFTP server.")    
    options: list[Options] = Field(default=None, description="DHCP options.")    
    server_type: Literal["regular", "ipsec"] | None = Field(default="regular", description="DHCP server can be a normal DHCP server or an IPsec DHCP server.")    
    ip_mode: Literal["range", "usrgrp"] | None = Field(default="range", description="Method used to assign client IP.")    
    conflicted_ip_timeout: int | None = Field(ge=60, le=8640000, default=1800, description="Time in seconds to wait after a conflicted IP address is removed from the DHCP range before it can be reused.")    
    ipsec_lease_hold: int | None = Field(ge=0, le=8640000, default=60, description="DHCP over IPsec leases expire this many seconds after tunnel down (0 to disable forced-expiry).")    
    auto_configuration: Literal["disable", "enable"] | None = Field(default="enable", description="Enable/disable auto configuration.")    
    dhcp_settings_from_fortiipam: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable populating of DHCP server settings from FortiIPAM.")    
    auto_managed_status: Literal["disable", "enable"] | None = Field(default="enable", description="Enable/disable use of this DHCP server once this interface has been assigned an IP address from FortiIPAM.")    
    ddns_update: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable DDNS update for DHCP.")    
    ddns_update_override: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable DDNS update override for DHCP.")    
    ddns_server_ip: str | None = Field(default="0.0.0.0", description="DDNS server IP.")    
    ddns_zone: str | None = Field(max_length=64, default="", description="Zone of your domain name (ex. DDNS.com).")    
    ddns_auth: Literal["disable", "tsig"] | None = Field(default="disable", description="DDNS authentication mode.")    
    ddns_keyname: str | None = Field(max_length=64, default="", description="DDNS update key name.")    
    ddns_key: Any = Field(default=None, description="DDNS update key (base 64 encoding).")    
    ddns_ttl: int | None = Field(ge=60, le=86400, default=300, description="TTL.")    
    vci_match: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable vendor class identifier (VCI) matching. When enabled only DHCP requests with a matching VCI are served.")    
    vci_string: list[VciString] = Field(default=None, description="One or more VCI strings in quotes separated by spaces.")    
    exclude_range: list[ExcludeRange] = Field(default=None, description="Exclude one or more ranges of IP addresses from being assigned to clients.")    
    shared_subnet: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable shared subnet.")    
    relay_agent: str | None = Field(default="0.0.0.0", description="Relay agent IP.")    
    reserved_address: list[ReservedAddress] = Field(default=None, description="Options for the DHCP server to assign IP settings to specific MAC addresses.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    @field_validator('interface')
    @classmethod
    def validate_interface(cls, v: Any) -> Any:
        """
        Validate interface field.
        
        Datasource: ['system.interface.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('timezone')
    @classmethod
    def validate_timezone(cls, v: Any) -> Any:
        """
        Validate timezone field.
        
        Datasource: ['system.timezone.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)
    # ========================================================================
    # Datasource Validation Methods
    # ========================================================================    
    async def validate_interface_references(self, client: Any) -> list[str]:
        """
        Validate interface references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/interface        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = ServerModel(
            ...     interface="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_interface_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.system.dhcp.server.post(policy.to_fortios_dict())
        """
        errors = []
        
        # Validate scalar field
        value = getattr(self, "interface", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.system.interface.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Interface '{value}' not found in "
                "system/interface"
            )        
        return errors    
    async def validate_timezone_references(self, client: Any) -> list[str]:
        """
        Validate timezone references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/timezone        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = ServerModel(
            ...     timezone="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_timezone_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.system.dhcp.server.post(policy.to_fortios_dict())
        """
        errors = []
        
        # Validate scalar field
        value = getattr(self, "timezone", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.system.timezone.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Timezone '{value}' not found in "
                "system/timezone"
            )        
        return errors    
    async def validate_all_references(self, client: Any) -> list[str]:
        """
        Validate ALL datasource references in this model.
        
        Convenience method that runs all validate_*_references() methods
        and aggregates the results.
        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of all validation errors found
            
        Example:
            >>> errors = await policy.validate_all_references(fgt._client)
            >>> if errors:
            ...     for error in errors:
            ...         print(f"  - {error}")
        """
        all_errors = []
        
        errors = await self.validate_interface_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_timezone_references(client)
        all_errors.extend(errors)        
        return all_errors

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "ServerModel",    "ServerIpRange",    "ServerTftpServer",    "ServerOptions",    "ServerVciString",    "ServerExcludeRange",    "ServerReservedAddress",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-09T09:47:41.566807Z
# ============================================================================


"""
Pydantic Models for CMDB - firewall/shaping_profile

Runtime validation models for firewall/shaping_profile configuration.
Generated from FortiOS schema version unknown.

Example Usage:
    >>> from hfortix_fortios.models.cmdb.firewall.shaping_profile import 
    >>>
    >>> # Create with validation
    >>> obj = (
    ...     profile-name=1,
    ...     name="example",
    ... )
    >>>
    >>> # Validation happens automatically
    >>> # ValidationError raised if constraints violated
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Literal, Optional

# ============================================================================
# Child Table Models
# ============================================================================

class ShapingProfileShapingEntries(BaseModel):
    """
    Child table model for shaping-entries.
    
    Define shaping entries of this shaping profile.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
    
    id: int | None = Field(ge=0, le=4294967295, default=0, description="ID number.")    
    class_id: int = Field(ge=0, le=4294967295, default=0, description="Class ID.")  # datasource: ['firewall.traffic-class.class-id']    
    priority: PriorityEnum | None = Field(default="high", description="Priority.")    
    guaranteed_bandwidth_percentage: int | None = Field(ge=0, le=100, default=0, description="Guaranteed bandwidth in percentage.")    
    maximum_bandwidth_percentage: int | None = Field(ge=1, le=100, default=1, description="Maximum bandwidth in percentage.")    
    limit: int | None = Field(ge=5, le=10000, default=100, description="Hard limit on the real queue size in packets.")    
    burst_in_msec: int | None = Field(ge=0, le=2000, default=0, description="Number of bytes that can be burst at maximum-bandwidth speed. Formula: burst = maximum-bandwidth*burst-in-msec.")    
    cburst_in_msec: int | None = Field(ge=0, le=2000, default=0, description="Number of bytes that can be burst as fast as the interface can transmit. Formula: cburst = maximum-bandwidth*cburst-in-msec.")    
    red_probability: int | None = Field(ge=0, le=20, default=0, description="Maximum probability (in percentage) for RED marking.")    
    min: int | None = Field(ge=3, le=3000, default=83, description="Average queue size in packets at which RED drop becomes a possibility.")    
    max: int | None = Field(ge=3, le=3000, default=250, description="Average queue size in packets at which RED drop probability is maximal.")
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================


# ============================================================================
# Main Model
# ============================================================================

class ShapingProfileModel(BaseModel):
    """
    Pydantic model for firewall/shaping_profile configuration.
    
    Configure shaping profiles.
    
    Validation Rules:        - profile_name: max_length=35 pattern=        - comment: max_length=1023 pattern=        - type: pattern=        - npu_offloading: pattern=        - default_class_id: min=0 max=4294967295 pattern=        - shaping_entries: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    profile_name: str = Field(max_length=35, default="", description="Shaping profile name.")    
    comment: str | None = Field(max_length=1023, default=None, description="Comment.")    
    type: Literal["policing", "queuing"] | None = Field(default="policing", description="Select shaping profile type: policing / queuing.")    
    npu_offloading: Literal["disable", "enable"] | None = Field(default="enable", description="Enable/disable NPU offloading.")    
    default_class_id: int = Field(ge=0, le=4294967295, default=0, description="Default class ID to handle unclassified packets (including all local traffic).")  # datasource: ['firewall.traffic-class.class-id']    
    shaping_entries: list[ShapingEntries] = Field(default=None, description="Define shaping entries of this shaping profile.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    @field_validator('default_class_id')
    @classmethod
    def validate_default_class_id(cls, v: Any) -> Any:
        """
        Validate default_class_id field.
        
        Datasource: ['firewall.traffic-class.class-id']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)
    # ========================================================================
    # Datasource Validation Methods
    # ========================================================================    
    async def validate_default_class_id_references(self, client: Any) -> list[str]:
        """
        Validate default_class_id references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - firewall/traffic-class        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = ShapingProfileModel(
            ...     default_class_id="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_default_class_id_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.firewall.shaping_profile.post(policy.to_fortios_dict())
        """
        errors = []
        
        # Validate scalar field
        value = getattr(self, "default_class_id", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.firewall.traffic-class.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Default-Class-Id '{value}' not found in "
                "firewall/traffic-class"
            )        
        return errors    
    async def validate_shaping_entries_references(self, client: Any) -> list[str]:
        """
        Validate shaping_entries references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - firewall/traffic-class        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = ShapingProfileModel(
            ...     shaping_entries=[{"class-id": "invalid-name"}],
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_shaping_entries_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.firewall.shaping_profile.post(policy.to_fortios_dict())
        """
        errors = []
        
        # Validate child table items
        values = getattr(self, "shaping_entries", [])
        if not values:
            return errors
        
        for item in values:
            if isinstance(item, dict):
                value = item.get("class-id")
            else:
                value = getattr(item, "class-id", None)
            
            if not value:
                continue
            
            # Check all datasource endpoints
            found = False
            if await client.api.cmdb.firewall.traffic-class.exists(value):
                found = True
            
            if not found:
                errors.append(
                    f"Shaping-Entries '{value}' not found in "
                    "firewall/traffic-class"
                )        
        return errors    
    async def validate_all_references(self, client: Any) -> list[str]:
        """
        Validate ALL datasource references in this model.
        
        Convenience method that runs all validate_*_references() methods
        and aggregates the results.
        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of all validation errors found
            
        Example:
            >>> errors = await policy.validate_all_references(fgt._client)
            >>> if errors:
            ...     for error in errors:
            ...         print(f"  - {error}")
        """
        all_errors = []
        
        errors = await self.validate_default_class_id_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_shaping_entries_references(client)
        all_errors.extend(errors)        
        return all_errors

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "ShapingProfileModel",    "ShapingProfileShapingEntries",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-09T09:47:41.493160Z
# ============================================================================


"""Validation helpers for vpn/ipsec/phase2 - Auto-generated"""

from typing import Any, TypedDict, NotRequired, Literal

# Import common validators from central _helpers module
from hfortix_fortios._helpers import (
    validate_enable_disable,
    validate_integer_range,
    validate_string_length,
    validate_port_number,
    validate_ip_address,
    validate_ipv6_address,
    validate_mac_address,
)

# Import central validation functions (avoid duplication across 1,062 files)
from hfortix_fortios._helpers.validation import (
    validate_required_fields as _validate_required_fields,
    validate_enum_field as _validate_enum_field,
    validate_query_parameter as _validate_query_parameter,
)

# ============================================================================
# Required Fields Validation
# Auto-generated from schema
# ============================================================================

# ⚠️  IMPORTANT: FortiOS schemas have known issues with required field marking:

# Do NOT use this list for strict validation - test with the actual FortiOS API!

# Fields marked as required (after filtering false positives)
REQUIRED_FIELDS = [
    "phase1name",  # Phase 1 determines the options required for phase 2.
    "proposal",  # Phase2 proposal.
    "src-name",  # Local proxy ID name.
    "src-name6",  # Local proxy ID name.
    "dst-name",  # Remote proxy ID name.
    "dst-name6",  # Remote proxy ID name.
]

# Fields with defaults (optional)
FIELDS_WITH_DEFAULTS = {
    "name": "",
    "phase1name": "",
    "dhcp-ipsec": "disable",
    "use-natip": "enable",
    "selector-match": "auto",
    "proposal": "",
    "pfs": "enable",
    "dhgrp": "20",
    "addke1": "",
    "addke2": "",
    "addke3": "",
    "addke4": "",
    "addke5": "",
    "addke6": "",
    "addke7": "",
    "replay": "enable",
    "keepalive": "disable",
    "auto-negotiate": "disable",
    "add-route": "phase1",
    "inbound-dscp-copy": "phase1",
    "keylifeseconds": 43200,
    "keylifekbs": 5120,
    "keylife-type": "seconds",
    "single-source": "disable",
    "route-overlap": "use-new",
    "encapsulation": "tunnel-mode",
    "l2tp": "disable",
    "initiator-ts-narrow": "disable",
    "diffserv": "disable",
    "diffservcode": "",
    "protocol": 0,
    "src-name": "",
    "src-name6": "",
    "src-addr-type": "subnet",
    "src-start-ip": "0.0.0.0",
    "src-start-ip6": "::",
    "src-end-ip": "0.0.0.0",
    "src-end-ip6": "::",
    "src-subnet": "0.0.0.0 0.0.0.0",
    "src-subnet6": "::/0",
    "src-port": 0,
    "dst-name": "",
    "dst-name6": "",
    "dst-addr-type": "subnet",
    "dst-start-ip": "0.0.0.0",
    "dst-start-ip6": "::",
    "dst-end-ip": "0.0.0.0",
    "dst-end-ip6": "::",
    "dst-subnet": "0.0.0.0 0.0.0.0",
    "dst-subnet6": "::/0",
    "dst-port": 0,
}

# ============================================================================
# Deprecated Fields
# Auto-generated from schema - warns users about deprecated fields
# ============================================================================

# Deprecated fields with migration guidance
DEPRECATED_FIELDS = {
}

# ============================================================================
# Field Metadata (Type Information & Descriptions)
# Auto-generated from schema - use for IDE autocomplete and documentation
# ============================================================================

# Field types mapping
FIELD_TYPES = {
    "name": "string",  # IPsec tunnel name.
    "phase1name": "string",  # Phase 1 determines the options required for phase 2.
    "dhcp-ipsec": "option",  # Enable/disable DHCP-IPsec.
    "use-natip": "option",  # Enable to use the FortiGate public IP as the source selector
    "selector-match": "option",  # Match type to use when comparing selectors.
    "proposal": "option",  # Phase2 proposal.
    "pfs": "option",  # Enable/disable PFS feature.
    "dhgrp": "option",  # Phase2 DH group.
    "addke1": "option",  # phase2 ADDKE1 group.
    "addke2": "option",  # phase2 ADDKE2 group.
    "addke3": "option",  # phase2 ADDKE3 group.
    "addke4": "option",  # phase2 ADDKE4 group.
    "addke5": "option",  # phase2 ADDKE5 group.
    "addke6": "option",  # phase2 ADDKE6 group.
    "addke7": "option",  # phase2 ADDKE7 group.
    "replay": "option",  # Enable/disable replay detection.
    "keepalive": "option",  # Enable/disable keep alive.
    "auto-negotiate": "option",  # Enable/disable IPsec SA auto-negotiation.
    "add-route": "option",  # Enable/disable automatic route addition.
    "inbound-dscp-copy": "option",  # Enable/disable copying of the DSCP in the ESP header to the 
    "keylifeseconds": "integer",  # Phase2 key life in time in seconds (120 - 172800).
    "keylifekbs": "integer",  # Phase2 key life in number of kilobytes of traffic (5120 - 42
    "keylife-type": "option",  # Keylife type.
    "single-source": "option",  # Enable/disable single source IP restriction.
    "route-overlap": "option",  # Action for overlapping routes.
    "encapsulation": "option",  # ESP encapsulation mode.
    "l2tp": "option",  # Enable/disable L2TP over IPsec.
    "comments": "var-string",  # Comment.
    "initiator-ts-narrow": "option",  # Enable/disable traffic selector narrowing for IKEv2 initiato
    "diffserv": "option",  # Enable/disable applying DSCP value to the IPsec tunnel outer
    "diffservcode": "user",  # DSCP value to be applied to the IPsec tunnel outer IP header
    "protocol": "integer",  # Quick mode protocol selector (1 - 255 or 0 for all).
    "src-name": "string",  # Local proxy ID name.
    "src-name6": "string",  # Local proxy ID name.
    "src-addr-type": "option",  # Local proxy ID type.
    "src-start-ip": "ipv4-address-any",  # Local proxy ID start.
    "src-start-ip6": "ipv6-address",  # Local proxy ID IPv6 start.
    "src-end-ip": "ipv4-address-any",  # Local proxy ID end.
    "src-end-ip6": "ipv6-address",  # Local proxy ID IPv6 end.
    "src-subnet": "ipv4-classnet-any",  # Local proxy ID subnet.
    "src-subnet6": "ipv6-prefix",  # Local proxy ID IPv6 subnet.
    "src-port": "integer",  # Quick mode source port (1 - 65535 or 0 for all).
    "dst-name": "string",  # Remote proxy ID name.
    "dst-name6": "string",  # Remote proxy ID name.
    "dst-addr-type": "option",  # Remote proxy ID type.
    "dst-start-ip": "ipv4-address-any",  # Remote proxy ID IPv4 start.
    "dst-start-ip6": "ipv6-address",  # Remote proxy ID IPv6 start.
    "dst-end-ip": "ipv4-address-any",  # Remote proxy ID IPv4 end.
    "dst-end-ip6": "ipv6-address",  # Remote proxy ID IPv6 end.
    "dst-subnet": "ipv4-classnet-any",  # Remote proxy ID IPv4 subnet.
    "dst-subnet6": "ipv6-prefix",  # Remote proxy ID IPv6 subnet.
    "dst-port": "integer",  # Quick mode destination port (1 - 65535 or 0 for all).
}

# Field descriptions (help text from FortiOS API)
FIELD_DESCRIPTIONS = {
    "name": "IPsec tunnel name.",
    "phase1name": "Phase 1 determines the options required for phase 2.",
    "dhcp-ipsec": "Enable/disable DHCP-IPsec.",
    "use-natip": "Enable to use the FortiGate public IP as the source selector when outbound NAT is used.",
    "selector-match": "Match type to use when comparing selectors.",
    "proposal": "Phase2 proposal.",
    "pfs": "Enable/disable PFS feature.",
    "dhgrp": "Phase2 DH group.",
    "addke1": "phase2 ADDKE1 group.",
    "addke2": "phase2 ADDKE2 group.",
    "addke3": "phase2 ADDKE3 group.",
    "addke4": "phase2 ADDKE4 group.",
    "addke5": "phase2 ADDKE5 group.",
    "addke6": "phase2 ADDKE6 group.",
    "addke7": "phase2 ADDKE7 group.",
    "replay": "Enable/disable replay detection.",
    "keepalive": "Enable/disable keep alive.",
    "auto-negotiate": "Enable/disable IPsec SA auto-negotiation.",
    "add-route": "Enable/disable automatic route addition.",
    "inbound-dscp-copy": "Enable/disable copying of the DSCP in the ESP header to the inner IP header.",
    "keylifeseconds": "Phase2 key life in time in seconds (120 - 172800).",
    "keylifekbs": "Phase2 key life in number of kilobytes of traffic (5120 - 4294967295).",
    "keylife-type": "Keylife type.",
    "single-source": "Enable/disable single source IP restriction.",
    "route-overlap": "Action for overlapping routes.",
    "encapsulation": "ESP encapsulation mode.",
    "l2tp": "Enable/disable L2TP over IPsec.",
    "comments": "Comment.",
    "initiator-ts-narrow": "Enable/disable traffic selector narrowing for IKEv2 initiator.",
    "diffserv": "Enable/disable applying DSCP value to the IPsec tunnel outer IP header.",
    "diffservcode": "DSCP value to be applied to the IPsec tunnel outer IP header.",
    "protocol": "Quick mode protocol selector (1 - 255 or 0 for all).",
    "src-name": "Local proxy ID name.",
    "src-name6": "Local proxy ID name.",
    "src-addr-type": "Local proxy ID type.",
    "src-start-ip": "Local proxy ID start.",
    "src-start-ip6": "Local proxy ID IPv6 start.",
    "src-end-ip": "Local proxy ID end.",
    "src-end-ip6": "Local proxy ID IPv6 end.",
    "src-subnet": "Local proxy ID subnet.",
    "src-subnet6": "Local proxy ID IPv6 subnet.",
    "src-port": "Quick mode source port (1 - 65535 or 0 for all).",
    "dst-name": "Remote proxy ID name.",
    "dst-name6": "Remote proxy ID name.",
    "dst-addr-type": "Remote proxy ID type.",
    "dst-start-ip": "Remote proxy ID IPv4 start.",
    "dst-start-ip6": "Remote proxy ID IPv6 start.",
    "dst-end-ip": "Remote proxy ID IPv4 end.",
    "dst-end-ip6": "Remote proxy ID IPv6 end.",
    "dst-subnet": "Remote proxy ID IPv4 subnet.",
    "dst-subnet6": "Remote proxy ID IPv6 subnet.",
    "dst-port": "Quick mode destination port (1 - 65535 or 0 for all).",
}

# Field constraints (string lengths, integer ranges)
FIELD_CONSTRAINTS = {
    "name": {"type": "string", "max_length": 35},
    "phase1name": {"type": "string", "max_length": 35},
    "keylifeseconds": {"type": "integer", "min": 120, "max": 172800},
    "keylifekbs": {"type": "integer", "min": 5120, "max": 4294967295},
    "protocol": {"type": "integer", "min": 0, "max": 255},
    "src-name": {"type": "string", "max_length": 79},
    "src-name6": {"type": "string", "max_length": 79},
    "src-port": {"type": "integer", "min": 0, "max": 65535},
    "dst-name": {"type": "string", "max_length": 79},
    "dst-name6": {"type": "string", "max_length": 79},
    "dst-port": {"type": "integer", "min": 0, "max": 65535},
}

# Nested schemas (for table/list fields with children)
NESTED_SCHEMAS = {
}


# Valid enum values from API documentation
VALID_BODY_DHCP_IPSEC = [
    "enable",
    "disable",
]
VALID_BODY_USE_NATIP = [
    "enable",
    "disable",
]
VALID_BODY_SELECTOR_MATCH = [
    "exact",
    "subset",
    "auto",
]
VALID_BODY_PROPOSAL = [
    "null-md5",
    "null-sha1",
    "null-sha256",
    "null-sha384",
    "null-sha512",
    "des-null",
    "des-md5",
    "des-sha1",
    "des-sha256",
    "des-sha384",
    "des-sha512",
    "3des-null",
    "3des-md5",
    "3des-sha1",
    "3des-sha256",
    "3des-sha384",
    "3des-sha512",
    "aes128-null",
    "aes128-md5",
    "aes128-sha1",
    "aes128-sha256",
    "aes128-sha384",
    "aes128-sha512",
    "aes128gcm",
    "aes192-null",
    "aes192-md5",
    "aes192-sha1",
    "aes192-sha256",
    "aes192-sha384",
    "aes192-sha512",
    "aes256-null",
    "aes256-md5",
    "aes256-sha1",
    "aes256-sha256",
    "aes256-sha384",
    "aes256-sha512",
    "aes256gcm",
    "chacha20poly1305",
    "aria128-null",
    "aria128-md5",
    "aria128-sha1",
    "aria128-sha256",
    "aria128-sha384",
    "aria128-sha512",
    "aria192-null",
    "aria192-md5",
    "aria192-sha1",
    "aria192-sha256",
    "aria192-sha384",
    "aria192-sha512",
    "aria256-null",
    "aria256-md5",
    "aria256-sha1",
    "aria256-sha256",
    "aria256-sha384",
    "aria256-sha512",
    "seed-null",
    "seed-md5",
    "seed-sha1",
    "seed-sha256",
    "seed-sha384",
    "seed-sha512",
]
VALID_BODY_PFS = [
    "enable",
    "disable",
]
VALID_BODY_DHGRP = [
    "1",
    "2",
    "5",
    "14",
    "15",
    "16",
    "17",
    "18",
    "19",
    "20",
    "21",
    "27",
    "28",
    "29",
    "30",
    "31",
    "32",
]
VALID_BODY_ADDKE1 = [
    "0",
    "35",
    "36",
    "37",
    "1080",
    "1081",
    "1082",
    "1083",
    "1084",
    "1085",
    "1089",
    "1090",
    "1091",
    "1092",
    "1093",
    "1094",
]
VALID_BODY_ADDKE2 = [
    "0",
    "35",
    "36",
    "37",
    "1080",
    "1081",
    "1082",
    "1083",
    "1084",
    "1085",
    "1089",
    "1090",
    "1091",
    "1092",
    "1093",
    "1094",
]
VALID_BODY_ADDKE3 = [
    "0",
    "35",
    "36",
    "37",
    "1080",
    "1081",
    "1082",
    "1083",
    "1084",
    "1085",
    "1089",
    "1090",
    "1091",
    "1092",
    "1093",
    "1094",
]
VALID_BODY_ADDKE4 = [
    "0",
    "35",
    "36",
    "37",
    "1080",
    "1081",
    "1082",
    "1083",
    "1084",
    "1085",
    "1089",
    "1090",
    "1091",
    "1092",
    "1093",
    "1094",
]
VALID_BODY_ADDKE5 = [
    "0",
    "35",
    "36",
    "37",
    "1080",
    "1081",
    "1082",
    "1083",
    "1084",
    "1085",
    "1089",
    "1090",
    "1091",
    "1092",
    "1093",
    "1094",
]
VALID_BODY_ADDKE6 = [
    "0",
    "35",
    "36",
    "37",
    "1080",
    "1081",
    "1082",
    "1083",
    "1084",
    "1085",
    "1089",
    "1090",
    "1091",
    "1092",
    "1093",
    "1094",
]
VALID_BODY_ADDKE7 = [
    "0",
    "35",
    "36",
    "37",
    "1080",
    "1081",
    "1082",
    "1083",
    "1084",
    "1085",
    "1089",
    "1090",
    "1091",
    "1092",
    "1093",
    "1094",
]
VALID_BODY_REPLAY = [
    "enable",
    "disable",
]
VALID_BODY_KEEPALIVE = [
    "enable",
    "disable",
]
VALID_BODY_AUTO_NEGOTIATE = [
    "enable",
    "disable",
]
VALID_BODY_ADD_ROUTE = [
    "phase1",
    "enable",
    "disable",
]
VALID_BODY_INBOUND_DSCP_COPY = [
    "phase1",
    "enable",
    "disable",
]
VALID_BODY_KEYLIFE_TYPE = [
    "seconds",
    "kbs",
    "both",
]
VALID_BODY_SINGLE_SOURCE = [
    "enable",
    "disable",
]
VALID_BODY_ROUTE_OVERLAP = [
    "use-old",
    "use-new",
    "allow",
]
VALID_BODY_ENCAPSULATION = [
    "tunnel-mode",
    "transport-mode",
]
VALID_BODY_L2TP = [
    "enable",
    "disable",
]
VALID_BODY_INITIATOR_TS_NARROW = [
    "enable",
    "disable",
]
VALID_BODY_DIFFSERV = [
    "enable",
    "disable",
]
VALID_BODY_SRC_ADDR_TYPE = [
    "subnet",
    "range",
    "ip",
    "name",
]
VALID_BODY_DST_ADDR_TYPE = [
    "subnet",
    "range",
    "ip",
    "name",
]
VALID_QUERY_ACTION = ["default", "schema"]

# ============================================================================
# GET Validation
# ============================================================================


def validate_vpn_ipsec_phase2_get(
    attr: str | None = None,
    filters: dict[str, Any] | None = None,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate GET request parameters for vpn/ipsec/phase2."""
    # Validate query parameters using central function
    if "action" in params:
        is_valid, error = _validate_query_parameter(
            "action",
            params.get("action"),
            VALID_QUERY_ACTION
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# POST Validation
# ============================================================================


def validate_vpn_ipsec_phase2_post(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate POST request to create new vpn/ipsec/phase2 object."""
    # Step 1: Validate required fields using central function
    is_valid, error = _validate_required_fields(
        payload,
        REQUIRED_FIELDS,
        FIELD_DESCRIPTIONS
    )
    if not is_valid:
        return (False, error)

    # Step 2: Validate enum values using central function
    if "dhcp-ipsec" in payload:
        is_valid, error = _validate_enum_field(
            "dhcp-ipsec",
            payload["dhcp-ipsec"],
            VALID_BODY_DHCP_IPSEC,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "use-natip" in payload:
        is_valid, error = _validate_enum_field(
            "use-natip",
            payload["use-natip"],
            VALID_BODY_USE_NATIP,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "selector-match" in payload:
        is_valid, error = _validate_enum_field(
            "selector-match",
            payload["selector-match"],
            VALID_BODY_SELECTOR_MATCH,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "proposal" in payload:
        is_valid, error = _validate_enum_field(
            "proposal",
            payload["proposal"],
            VALID_BODY_PROPOSAL,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "pfs" in payload:
        is_valid, error = _validate_enum_field(
            "pfs",
            payload["pfs"],
            VALID_BODY_PFS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "dhgrp" in payload:
        is_valid, error = _validate_enum_field(
            "dhgrp",
            payload["dhgrp"],
            VALID_BODY_DHGRP,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "addke1" in payload:
        is_valid, error = _validate_enum_field(
            "addke1",
            payload["addke1"],
            VALID_BODY_ADDKE1,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "addke2" in payload:
        is_valid, error = _validate_enum_field(
            "addke2",
            payload["addke2"],
            VALID_BODY_ADDKE2,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "addke3" in payload:
        is_valid, error = _validate_enum_field(
            "addke3",
            payload["addke3"],
            VALID_BODY_ADDKE3,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "addke4" in payload:
        is_valid, error = _validate_enum_field(
            "addke4",
            payload["addke4"],
            VALID_BODY_ADDKE4,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "addke5" in payload:
        is_valid, error = _validate_enum_field(
            "addke5",
            payload["addke5"],
            VALID_BODY_ADDKE5,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "addke6" in payload:
        is_valid, error = _validate_enum_field(
            "addke6",
            payload["addke6"],
            VALID_BODY_ADDKE6,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "addke7" in payload:
        is_valid, error = _validate_enum_field(
            "addke7",
            payload["addke7"],
            VALID_BODY_ADDKE7,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "replay" in payload:
        is_valid, error = _validate_enum_field(
            "replay",
            payload["replay"],
            VALID_BODY_REPLAY,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "keepalive" in payload:
        is_valid, error = _validate_enum_field(
            "keepalive",
            payload["keepalive"],
            VALID_BODY_KEEPALIVE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "auto-negotiate" in payload:
        is_valid, error = _validate_enum_field(
            "auto-negotiate",
            payload["auto-negotiate"],
            VALID_BODY_AUTO_NEGOTIATE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "add-route" in payload:
        is_valid, error = _validate_enum_field(
            "add-route",
            payload["add-route"],
            VALID_BODY_ADD_ROUTE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "inbound-dscp-copy" in payload:
        is_valid, error = _validate_enum_field(
            "inbound-dscp-copy",
            payload["inbound-dscp-copy"],
            VALID_BODY_INBOUND_DSCP_COPY,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "keylife-type" in payload:
        is_valid, error = _validate_enum_field(
            "keylife-type",
            payload["keylife-type"],
            VALID_BODY_KEYLIFE_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "single-source" in payload:
        is_valid, error = _validate_enum_field(
            "single-source",
            payload["single-source"],
            VALID_BODY_SINGLE_SOURCE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "route-overlap" in payload:
        is_valid, error = _validate_enum_field(
            "route-overlap",
            payload["route-overlap"],
            VALID_BODY_ROUTE_OVERLAP,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "encapsulation" in payload:
        is_valid, error = _validate_enum_field(
            "encapsulation",
            payload["encapsulation"],
            VALID_BODY_ENCAPSULATION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "l2tp" in payload:
        is_valid, error = _validate_enum_field(
            "l2tp",
            payload["l2tp"],
            VALID_BODY_L2TP,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "initiator-ts-narrow" in payload:
        is_valid, error = _validate_enum_field(
            "initiator-ts-narrow",
            payload["initiator-ts-narrow"],
            VALID_BODY_INITIATOR_TS_NARROW,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "diffserv" in payload:
        is_valid, error = _validate_enum_field(
            "diffserv",
            payload["diffserv"],
            VALID_BODY_DIFFSERV,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "src-addr-type" in payload:
        is_valid, error = _validate_enum_field(
            "src-addr-type",
            payload["src-addr-type"],
            VALID_BODY_SRC_ADDR_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "dst-addr-type" in payload:
        is_valid, error = _validate_enum_field(
            "dst-addr-type",
            payload["dst-addr-type"],
            VALID_BODY_DST_ADDR_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# PUT Validation
# ============================================================================


def validate_vpn_ipsec_phase2_put(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate PUT request to update vpn/ipsec/phase2."""
    # Validate enum values using central function
    if "dhcp-ipsec" in payload:
        is_valid, error = _validate_enum_field(
            "dhcp-ipsec",
            payload["dhcp-ipsec"],
            VALID_BODY_DHCP_IPSEC,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "use-natip" in payload:
        is_valid, error = _validate_enum_field(
            "use-natip",
            payload["use-natip"],
            VALID_BODY_USE_NATIP,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "selector-match" in payload:
        is_valid, error = _validate_enum_field(
            "selector-match",
            payload["selector-match"],
            VALID_BODY_SELECTOR_MATCH,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "proposal" in payload:
        is_valid, error = _validate_enum_field(
            "proposal",
            payload["proposal"],
            VALID_BODY_PROPOSAL,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "pfs" in payload:
        is_valid, error = _validate_enum_field(
            "pfs",
            payload["pfs"],
            VALID_BODY_PFS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "dhgrp" in payload:
        is_valid, error = _validate_enum_field(
            "dhgrp",
            payload["dhgrp"],
            VALID_BODY_DHGRP,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "addke1" in payload:
        is_valid, error = _validate_enum_field(
            "addke1",
            payload["addke1"],
            VALID_BODY_ADDKE1,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "addke2" in payload:
        is_valid, error = _validate_enum_field(
            "addke2",
            payload["addke2"],
            VALID_BODY_ADDKE2,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "addke3" in payload:
        is_valid, error = _validate_enum_field(
            "addke3",
            payload["addke3"],
            VALID_BODY_ADDKE3,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "addke4" in payload:
        is_valid, error = _validate_enum_field(
            "addke4",
            payload["addke4"],
            VALID_BODY_ADDKE4,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "addke5" in payload:
        is_valid, error = _validate_enum_field(
            "addke5",
            payload["addke5"],
            VALID_BODY_ADDKE5,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "addke6" in payload:
        is_valid, error = _validate_enum_field(
            "addke6",
            payload["addke6"],
            VALID_BODY_ADDKE6,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "addke7" in payload:
        is_valid, error = _validate_enum_field(
            "addke7",
            payload["addke7"],
            VALID_BODY_ADDKE7,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "replay" in payload:
        is_valid, error = _validate_enum_field(
            "replay",
            payload["replay"],
            VALID_BODY_REPLAY,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "keepalive" in payload:
        is_valid, error = _validate_enum_field(
            "keepalive",
            payload["keepalive"],
            VALID_BODY_KEEPALIVE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "auto-negotiate" in payload:
        is_valid, error = _validate_enum_field(
            "auto-negotiate",
            payload["auto-negotiate"],
            VALID_BODY_AUTO_NEGOTIATE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "add-route" in payload:
        is_valid, error = _validate_enum_field(
            "add-route",
            payload["add-route"],
            VALID_BODY_ADD_ROUTE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "inbound-dscp-copy" in payload:
        is_valid, error = _validate_enum_field(
            "inbound-dscp-copy",
            payload["inbound-dscp-copy"],
            VALID_BODY_INBOUND_DSCP_COPY,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "keylife-type" in payload:
        is_valid, error = _validate_enum_field(
            "keylife-type",
            payload["keylife-type"],
            VALID_BODY_KEYLIFE_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "single-source" in payload:
        is_valid, error = _validate_enum_field(
            "single-source",
            payload["single-source"],
            VALID_BODY_SINGLE_SOURCE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "route-overlap" in payload:
        is_valid, error = _validate_enum_field(
            "route-overlap",
            payload["route-overlap"],
            VALID_BODY_ROUTE_OVERLAP,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "encapsulation" in payload:
        is_valid, error = _validate_enum_field(
            "encapsulation",
            payload["encapsulation"],
            VALID_BODY_ENCAPSULATION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "l2tp" in payload:
        is_valid, error = _validate_enum_field(
            "l2tp",
            payload["l2tp"],
            VALID_BODY_L2TP,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "initiator-ts-narrow" in payload:
        is_valid, error = _validate_enum_field(
            "initiator-ts-narrow",
            payload["initiator-ts-narrow"],
            VALID_BODY_INITIATOR_TS_NARROW,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "diffserv" in payload:
        is_valid, error = _validate_enum_field(
            "diffserv",
            payload["diffserv"],
            VALID_BODY_DIFFSERV,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "src-addr-type" in payload:
        is_valid, error = _validate_enum_field(
            "src-addr-type",
            payload["src-addr-type"],
            VALID_BODY_SRC_ADDR_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "dst-addr-type" in payload:
        is_valid, error = _validate_enum_field(
            "dst-addr-type",
            payload["dst-addr-type"],
            VALID_BODY_DST_ADDR_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# Metadata Access Functions
# Imported from central module to avoid duplication across 1,062 files
# ============================================================================

from hfortix_fortios._helpers.metadata import (
    get_field_description as _get_field_description,
    get_field_type as _get_field_type,
    get_field_constraints as _get_field_constraints,
    get_field_default as _get_field_default,
    get_field_options as _get_field_options,
    get_nested_schema as _get_nested_schema,
    get_all_fields as _get_all_fields,
    get_field_metadata as _get_field_metadata,
    validate_field_value as _validate_field_value,
)

# Wrapper functions that bind module-specific data to central functions
def get_field_description(field_name: str) -> str | None:
    """Get description/help text for a field."""
    return _get_field_description(FIELD_DESCRIPTIONS, field_name)

def get_field_type(field_name: str) -> str | None:
    """Get the type of a field."""
    return _get_field_type(FIELD_TYPES, field_name)

def get_field_constraints(field_name: str) -> dict[str, Any] | None:
    """Get constraints for a field (min/max values, string length)."""
    return _get_field_constraints(FIELD_CONSTRAINTS, field_name)

def get_field_default(field_name: str) -> Any | None:
    """Get default value for a field."""
    return _get_field_default(FIELDS_WITH_DEFAULTS, field_name)

def get_field_options(field_name: str) -> list[str] | None:
    """Get valid enum options for a field."""
    return _get_field_options(globals(), field_name)

def get_nested_schema(field_name: str) -> dict[str, Any] | None:
    """Get schema for nested table/list fields."""
    return _get_nested_schema(NESTED_SCHEMAS, field_name)

def get_all_fields() -> list[str]:
    """Get list of all field names."""
    return _get_all_fields(FIELD_TYPES)

def get_field_metadata(field_name: str) -> dict[str, Any] | None:
    """Get complete metadata for a field (type, description, constraints, defaults, options)."""
    return _get_field_metadata(
        FIELD_TYPES, FIELD_DESCRIPTIONS, FIELD_CONSTRAINTS,
        FIELDS_WITH_DEFAULTS, REQUIRED_FIELDS, NESTED_SCHEMAS,
        globals(), field_name
    )

def validate_field_value(field_name: str, value: Any) -> tuple[bool, str | None]:
    """Validate a single field value against its constraints."""
    return _validate_field_value(
        FIELD_TYPES, FIELD_DESCRIPTIONS, FIELD_CONSTRAINTS,
        globals(), field_name, value
    )


# ============================================================================
# Schema Information
# Metadata about this endpoint schema
# ============================================================================

SCHEMA_INFO = {
    "endpoint": "vpn/ipsec/phase2",
    "category": "cmdb",
    "api_path": "vpn.ipsec/phase2",
    "mkey": "name",
    "mkey_type": "string",
    "help": "Configure VPN autokey tunnel.",
    "total_fields": 52,
    "required_fields_count": 6,
    "fields_with_defaults_count": 51,
}


def get_schema_info() -> dict[str, Any]:
    """Get information about this endpoint schema."""
    return SCHEMA_INFO.copy()



"""Type stubs for PROXY category."""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from hfortix_core.http.interface import IHTTPClient
    from .sessions import Sessions, SessionsDictMode, SessionsObjectMode

__all__ = [
    "Sessions",
    "ProxyDictMode",
    "ProxyObjectMode",
]

class ProxyDictMode:
    """PROXY API category for dict response mode.
    
    This class is returned when the client is instantiated with response_mode="dict" (default).
    All endpoints return dict/TypedDict responses by default.
    """
    
    sessions: SessionsDictMode

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize proxy category with HTTP client."""
        ...


class ProxyObjectMode:
    """PROXY API category for object response mode.
    
    This class is returned when the client is instantiated with response_mode="object".
    All endpoints return FortiObject responses by default.
    """
    
    sessions: SessionsObjectMode

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize proxy category with HTTP client."""
        ...


# Base class for backwards compatibility
class Proxy:
    """PROXY API category."""
    
    sessions: Sessions

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize proxy category with HTTP client."""
        ...



"""FortiOS CMDB - Syslogd2 category"""

from .filter import Filter
from .override_filter import OverrideFilter
from .override_setting import OverrideSetting
from .setting import Setting

__all__ = [
    "Filter",
    "OverrideFilter",
    "OverrideSetting",
    "Setting",
    "Syslogd2",
]


class Syslogd2:
    """Syslogd2 endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """Syslogd2 endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.filter = Filter(client)
        self.override_filter = OverrideFilter(client)
        self.override_setting = OverrideSetting(client)
        self.setting = Setting(client)



"""
Pydantic Models for CMDB - dlp/dictionary

Runtime validation models for dlp/dictionary configuration.
Generated from FortiOS schema version unknown.

Example Usage:
    >>> from hfortix_fortios.models.cmdb.dlp.dictionary import 
    >>>
    >>> # Create with validation
    >>> obj = (
    ...     name=1,
    ...     name="example",
    ... )
    >>>
    >>> # Validation happens automatically
    >>> # ValidationError raised if constraints violated
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Literal, Optional
from uuid import UUID

# ============================================================================
# Child Table Models
# ============================================================================

class DictionaryEntries(BaseModel):
    """
    Child table model for entries.
    
    DLP dictionary entries.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
    
    id: int | None = Field(ge=0, le=4294967295, default=0, description="ID.")    
    type: str = Field(max_length=35, default="", description="Pattern type to match.")  # datasource: ['dlp.data-type.name']    
    pattern: str = Field(max_length=255, default="", description="Pattern to match.")    
    ignore_case: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable ignore case.")    
    repeat: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable repeat match.")    
    status: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable this pattern.")    
    comment: str | None = Field(max_length=255, default=None, description="Optional comments.")
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================


# ============================================================================
# Main Model
# ============================================================================

class DictionaryModel(BaseModel):
    """
    Pydantic model for dlp/dictionary configuration.
    
    Configure dictionaries used by DLP blocking.
    
    Validation Rules:        - uuid: pattern=        - name: max_length=35 pattern=        - match_type: pattern=        - match_around: pattern=        - comment: max_length=255 pattern=        - entries: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    uuid: str | None = Field(default="00000000-0000-0000-0000-000000000000", description="Universally Unique Identifier (UUID; automatically assigned but can be manually reset).")    
    name: str = Field(max_length=35, default="", description="Name of table containing the dictionary.")    
    match_type: Literal["match-all", "match-any"] = Field(default="match-any", description="Logical relation between entries (default = match-any).")    
    match_around: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable match-around support.")    
    comment: str | None = Field(max_length=255, default=None, description="Optional comments.")    
    entries: list[Entries] = Field(description="DLP dictionary entries.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)
    # ========================================================================
    # Datasource Validation Methods
    # ========================================================================    
    async def validate_entries_references(self, client: Any) -> list[str]:
        """
        Validate entries references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - dlp/data-type        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = DictionaryModel(
            ...     entries=[{"type": "invalid-name"}],
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_entries_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.dlp.dictionary.post(policy.to_fortios_dict())
        """
        errors = []
        
        # Validate child table items
        values = getattr(self, "entries", [])
        if not values:
            return errors
        
        for item in values:
            if isinstance(item, dict):
                value = item.get("type")
            else:
                value = getattr(item, "type", None)
            
            if not value:
                continue
            
            # Check all datasource endpoints
            found = False
            if await client.api.cmdb.dlp.data-type.exists(value):
                found = True
            
            if not found:
                errors.append(
                    f"Entries '{value}' not found in "
                    "dlp/data-type"
                )        
        return errors    
    async def validate_all_references(self, client: Any) -> list[str]:
        """
        Validate ALL datasource references in this model.
        
        Convenience method that runs all validate_*_references() methods
        and aggregates the results.
        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of all validation errors found
            
        Example:
            >>> errors = await policy.validate_all_references(fgt._client)
            >>> if errors:
            ...     for error in errors:
            ...         print(f"  - {error}")
        """
        all_errors = []
        
        errors = await self.validate_entries_references(client)
        all_errors.extend(errors)        
        return all_errors

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "DictionaryModel",    "DictionaryEntries",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-09T09:47:41.628650Z
# ============================================================================


"""Type stubs for FORTIGUARD category."""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from hfortix_core.http.interface import IHTTPClient
    from .clear_statistics import ClearStatistics, ClearStatisticsDictMode, ClearStatisticsObjectMode
    from .manual_update import ManualUpdate, ManualUpdateDictMode, ManualUpdateObjectMode
    from .server_info import ServerInfo, ServerInfoDictMode, ServerInfoObjectMode
    from .test_availability import TestAvailability, TestAvailabilityDictMode, TestAvailabilityObjectMode
    from .update import Update, UpdateDictMode, UpdateObjectMode

__all__ = [
    "ClearStatistics",
    "ManualUpdate",
    "ServerInfo",
    "TestAvailability",
    "Update",
    "FortiguardDictMode",
    "FortiguardObjectMode",
]

class FortiguardDictMode:
    """FORTIGUARD API category for dict response mode.
    
    This class is returned when the client is instantiated with response_mode="dict" (default).
    All endpoints return dict/TypedDict responses by default.
    """
    
    clear_statistics: ClearStatisticsDictMode
    manual_update: ManualUpdateDictMode
    server_info: ServerInfoDictMode
    test_availability: TestAvailabilityDictMode
    update: UpdateDictMode

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize fortiguard category with HTTP client."""
        ...


class FortiguardObjectMode:
    """FORTIGUARD API category for object response mode.
    
    This class is returned when the client is instantiated with response_mode="object".
    All endpoints return FortiObject responses by default.
    """
    
    clear_statistics: ClearStatisticsObjectMode
    manual_update: ManualUpdateObjectMode
    server_info: ServerInfoObjectMode
    test_availability: TestAvailabilityObjectMode
    update: UpdateObjectMode

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize fortiguard category with HTTP client."""
        ...


# Base class for backwards compatibility
class Fortiguard:
    """FORTIGUARD API category."""
    
    clear_statistics: ClearStatistics
    manual_update: ManualUpdate
    server_info: ServerInfo
    test_availability: TestAvailability
    update: Update

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize fortiguard category with HTTP client."""
        ...
